/************************************************************************
 *
 *	NoteConverter.java
 *
 *  Copyright: 2002-2015 by Henrik Just
 *
 *  This file is part of Writer2LaTeX.
 *  
 *  Writer2LaTeX is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  Writer2LaTeX is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with Writer2LaTeX.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Version 1.6 (2015-06-14)
 *
 */
package w2phtml.xhtml.content;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import w2phtml.office.OfficeReader;
import w2phtml.office.PropertySet;
import w2phtml.office.XMLString;
import w2phtml.util.Misc;
import w2phtml.xhtml.Converter;
import w2phtml.xhtml.Parser;
import w2phtml.xhtml.StyleInfo;
import w2phtml.xhtml.XhtmlConfig;

/** This is a base class handles the conversion of footnotes and endnotes
 */
class NoteParser extends Parser {
	
	// The notes configuration
	private PropertySet noteConfig;
	
	// The collection of notes
	//private List<Node> notes = new ArrayList<Node>();
	private HashMap<String, List<Node>> notes = new HashMap<String, List<Node>>();
	/** Construct a new note converter
	 * 
	 * @param ofr the office reader used to read the source document
	 * @param config the configuration
	 * @param converter the converter
	 * @param noteConfig the configuration of the notes
	 */
    NoteParser(OfficeReader ofr, XhtmlConfig config, Converter converter, PropertySet noteConfig) {
        super(ofr,config,converter);
        this.noteConfig = noteConfig;
    }
    
    /** Handle a footnote or endnote. This method inserts the citation and stores the actual note for later processing
     * 
     * @param onode a text:note element
     * @param hnode the inline HTML element to contain the citation
     */
     void handleNote(Node onode, Node hnode, String section) {
    	// Create a style span for the citation
        String sCitBodyStyle = noteConfig.getProperty(XMLString.TEXT_CITATION_BODY_STYLE_NAME);
		Element span = getTextParser().createInline((Element) hnode,sCitBodyStyle);
        // Add target and back-link to the span
        String sId = Misc.getAttribute(onode,XMLString.TEXT_ID);
        Element link = converter.createLink(sId);
        converter.addTarget(link,"body"+sId);
        converter.addEpubType(link, "noteref");
		span.appendChild(link);
		// Get the citation
        Element citation = Misc.getChildByTagName(onode,XMLString.TEXT_NOTE_CITATION);
        if (citation==null) { // try old format
	        citation = Misc.getChildByTagName(onode,XMLString.TEXT_FOOTNOTE_CITATION);
	        if (citation==null) {
	        	citation = Misc.getChildByTagName(onode,XMLString.TEXT_ENDNOTE_CITATION);
	        }
        }
        // Insert the citation
        if (citation!=null) {
        	getTextParser().traversePCDATA(citation,link);
        }
        // Remember the actual note
        List<Node> noteList = notes.get(section);
        if (noteList == null) {
        	noteList = new ArrayList<Node>();
        	notes.put(section, noteList);
        }
      	noteList.add(onode);
	}
    
    boolean hasNotes() {
    	return notes.size()>0;
    }
    
    Element createNoteSection(Node hnode, String sEpubType) {
    	Element section = converter.createElement(converter.isHTML5() ? "section" : "div");
    	hnode.appendChild(section);
    	converter.addEpubType(section, sEpubType);
    	return section;
    }
     
    void insertNoteHeading(Node hnode, String sHeading, String sTarget) {
    	if (sHeading.length()>0) {
    		// Create heading
    		Element heading = converter.createElement("h1");
    		hnode.appendChild(heading);
    		heading.appendChild(converter.createTextNode(sHeading));

    		// Add to external content.
    		if (config.getXhtmlSplitLevel()>0) {
            	converter.addContentEntry(sHeading, 1, null);        			
    		}
    		else {
    			//For single output file we need a target
                converter.addTarget(heading,sTarget);                
            	converter.addContentEntry(sHeading, 1, sTarget);        			
    		}
    	}
    }
    public void flushAllNotes(Node hnode, String sEpubType) {
    	for (Entry<String, List<Node>> entry : notes.entrySet()) {
    		 flushNotes(hnode, sEpubType, entry.getKey());
			}
    }
    
    public void flushNotes(Node hnode, String sEpubType, String section) {
    	List<Node> noteList = notes.get(section);
    	if (noteList == null) {
    		//System.out.println("noteList is null");
    		//System.out.println("notes " + notes.size());
  			/*for (StackTraceElement ste : Thread.currentThread().getStackTrace()) {
  				System.out.println(ste);
  			}*/
    		return;
    	}
    	
    	int nSize = noteList.size();
		for (int i=0; i<nSize; i++) {
			Node note = noteList.get(i);
			// Create container
			Element aside = converter.createElement(converter.isHTML5() ? "aside" : "div");
			hnode.appendChild(aside);
			converter.addEpubType(aside, sEpubType);
			// Get the citation
			Node citation = Misc.getChildByTagName(note,XMLString.TEXT_NOTE_CITATION);
			if (citation==null) { // try old format
				citation = Misc.getChildByTagName(note,XMLString.TEXT_FOOTNOTE_CITATION);
				if (citation==null) {
					citation = Misc.getChildByTagName(note,XMLString.TEXT_ENDNOTE_CITATION);
				}
			}
			// Get the body
			Node body = Misc.getChildByTagName(note,XMLString.TEXT_NOTE_BODY);
			if (body==null) { // try old format
				body = Misc.getChildByTagName(note,XMLString.TEXT_FOOTNOTE_BODY);
				if (body==null) {
					body = Misc.getChildByTagName(note,XMLString.TEXT_ENDNOTE_BODY);
				}
			}
			// Export the note
			String sId = Misc.getAttribute(note,XMLString.TEXT_ID); 
	        converter.addTarget(aside,sId);
			createAnchor(sId,citation);
	        getTextParser().traverseBlockText(body,aside);
		}
		//noteList.clear();
		notes.remove(section);
    }

    private void createAnchor(String sId, Node citation) {
        // Create target and link
        Element link = converter.createLink("body"+sId);

        // Style it
        String sCitStyle = noteConfig.getProperty(XMLString.TEXT_CITATION_STYLE_NAME);
        StyleInfo linkInfo = new StyleInfo();
        getTextSP().readStyle(sCitStyle,linkInfo);
        writeStyle(linkInfo,link);

        // Add prefix
        String sPrefix = noteConfig.getProperty(XMLString.STYLE_NUM_PREFIX);
        if (sPrefix!=null) {
        	link.appendChild(converter.createTextNode(sPrefix));
        }

        // Add citation
        getTextParser().traversePCDATA(citation,link);

        // Add suffix
        String sSuffix = noteConfig.getProperty(XMLString.STYLE_NUM_SUFFIX);
        if (sSuffix!=null) {
        	link.appendChild(converter.createTextNode(sSuffix));        	
        }

        // Add space
        Element span = converter.createElement("span");
        span.appendChild(link);
        span.appendChild(converter.createTextNode(" "));

        // Save it for later insertion
        getTextParser().setAsapNode(span);
    }
    
}
